﻿using System;
using System.Diagnostics.Contracts;

namespace CodeContractsExample
{
    public class Teacher
    {
        public String Name { get; set; }

        public int Salary { get; set; }

        // http://msdn.microsoft.com/en-us/library/dd412886(v=vs.110).aspx
        [ContractInvariantMethod]
        private void ObjectInvariant()
        {
            Contract.Invariant(Salary >= 0);
            Contract.Invariant(!String.IsNullOrEmpty(Name));
        }

        public Teacher(String name)
        {
            Name = name;
        }

        public void RaiseSalaryTo(int amount)
        {
            Contract.Requires(amount > Salary); // Pre
            Contract.Ensures(Salary > Contract.OldValue<int>(Salary)); // Post
            Salary = amount;
        }

        public void LowerSalaryTo(int amount)
        {
            Contract.Requires<ArgumentOutOfRangeException>(amount < Salary);
            Contract.EnsuresOnThrow<InvalidOperationException>(Salary < Contract.OldValue<int>(Salary));
            Salary = amount;
        }      
    }
}
